#include "config.h"
#include <gnome.h>
#include <gtk/gtk.h>
#include "dnd_main.h"
#include "dnd_command.h"
#include "dnd_filesel.h"
#include "dnd_glarea.h"
#include "molecule.h"
#include "dnd_graphics.h"

void
file_open_cb (GtkWidget *widget, gpointer gdata)
{
  DNDData *data = gdata;
  gboolean ok;
  
  ok = file_open_dialog (widget, data);

  if (!ok)
    return;
  
  if (data->mol != NULL)
    {
      molecule_free (&data->mol);
      delete_display_lists ();
    }
  
  ok = read_psf (&data->mol, data->psf_name);

  if (!ok)
    return;

  ok = FALSE;
  
  switch (data->file_format)
    {
    case DND_PDB_FORMAT:
      ok = read_pdb (&data->mol, data->file_name);
      break;
    case DND_CRD_FORMAT:
      ok = read_crd (&data->mol, data->file_name);
      break;
    }
  
  if (!ok)
    return;
    
  if (data->mol != NULL)
    dnd_display (data);
    
  gtk_widget_draw (GTK_WIDGET (data->glarea), NULL);
  
  /* [FIXME] */  

  return;
}


void
file_save_cb (GtkWidget *widget, gpointer data)
{
  g_return_if_fail (data != NULL);
  
  file_save (data);
}

void
file_save_as_cb (GtkWidget *widget, gpointer data)
{
  g_return_if_fail (data != NULL);
  
  file_save_as_dialog (data);
}


void
file_close_cb (GtkWidget *widget, gpointer gdata)
{
  DNDData *data = gdata;
  
  molecule_free (&data->mol);
  delete_display_lists ();

  gtk_widget_draw (GTK_WIDGET (data->glarea), NULL);
}


void
file_quit_cb (GtkWidget *widget, void *data)
{
  gtk_main_quit ();
  return;
}


void
about_cb (GtkWidget *widget, void *data)
{
  GtkWidget *about;
  const gchar *authors[] = {
    "SooHaeng Yoo", NULL
  };
  
  about = gnome_about_new (_("dnd package"), VERSION,
			   _("(C) 1999 the Free Software Foundation"),
			   authors,
			   /* another comments */
			   _("DND package is a GUI program "
			     "for using Molecular Dynamics "),
			   NULL);
  gtk_widget_show (about);
  
  return;
}

void
popup_rotate_cb (GtkWidget *widget, gpointer gdata)
{
  mouse_option = ROTATE_MODE;
}


void
popup_translate_cb (GtkWidget *widget, gpointer gdata)
{
  mouse_option = TRANSLATE_MODE;
}


void
popup_zoom_cb (GtkWidget *widget, gpointer gdata)
{
  mouse_option = ZOOM_MODE;
}


void
popup_pick_cb (GtkWidget *widget, gpointer gdata)
{
  mouse_option = PICK_MODE;
}


void
popup_perspective_cb (GtkWidget *widget, gpointer gdata)
{
  DNDData *data = gdata;
  
  project_option = PERSPECTIVE_MODE;

  gtk_widget_draw (GTK_WIDGET (data->glarea), NULL);
  
}


void
popup_orthographic_cb (GtkWidget *widget, gpointer gdata)
{
  DNDData *data = gdata;
  
  project_option = ORTHO_MODE;

  gtk_widget_draw (GTK_WIDGET (data->glarea), NULL);
}


/*
void
set_color_cb (GtkWidget *widget, gpointer gdata)
{
  DNDData *data = gdata;
  
  set_color_dialog (gdata, data->color_info);
}
*/


void
setting_graphics_ball_stick_cb (GtkWidget *widget, gpointer gdata)
{
  DNDData *data = gdata;

  
  
}

void
setting_graphics_stick_cb (GtkWidget *widget, gpointer gdata)
{
  DNDData *data = gdata;

  data->graphics_info->graphics_mode = DND_GRAPHICS_CYLINDER;

  if (data->mol != NULL)
    {
      delete_display_lists ();
      dnd_display (data);
      gtk_widget_draw (GTK_WIDGET (data->glarea), NULL);
    }
}


void
setting_graphics_bond_cb (GtkWidget *widget, gpointer gdata)
{
  DNDData *data = gdata;

  data->graphics_info->graphics_mode = DND_GRAPHICS_BOND;

  if (data->mol != NULL)
    {
      delete_display_lists ();
      dnd_display (data);
      gtk_widget_draw (GTK_WIDGET (data->glarea), NULL);
    }
  
}
