#include <gnome.h>
#include <string.h>
#include "dnd_main.h"
#include "dnd_filesel.h"
#include "dnd_util.h"
#include "dnd_radio.h"


static struct 
{
  gchar *name;
  gint  file_format;
} format_item [] =
{
  { "PDB", DND_PDB_FORMAT},
  { "CRD", DND_CRD_FORMAT},
  { NULL, DND_NONE_FORMAT}    
};

static GList *dnd_file_savers;
static FileSaver *current_saver;


static void
set_ok (GtkWidget *widget, gboolean *dialog_result)
{
  *dialog_result = TRUE;
  gtk_main_quit ();
}


static void
saver_activate (GtkMenuItem *item, FileSaver *saver)
{
  GList *l;
  
  for (l = dnd_file_savers; l; l = l->next)
    {
      FileSaver *fs = l->data;
      
      if (fs == saver)
	current_saver = saver;
    }
  
}


static gboolean
file_saver_is_default_format (FileSaver *saver)
{
  if (current_saver == saver)
    return TRUE;
  return FALSE;
}


static void
fill_save_menu (GtkOptionMenu *omenu, GtkMenu *menu)
{
  GList *l;
  int i;
  FileSaver *fs;
  
  for (i = 0, l = dnd_file_savers; l; l = l->next, i++)
    {
      GtkWidget *menu_item;
  
      fs = l->data;
      
      menu_item =
	gtk_menu_item_new_with_label (fs->format_description);
      
      gtk_widget_show (menu_item);
      gtk_menu_append (menu, menu_item);
    
      if (file_saver_is_default_format (fs))
	gtk_option_menu_set_history (omenu, i);
    
      gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
			  GTK_SIGNAL_FUNC(saver_activate), fs);
    }
}


static GtkWidget *
make_format_chooser (void)
{
  GtkWidget *box, *label;
  GtkWidget *omenu, *menu;

  box = gtk_hbox_new (0, GNOME_PAD);
  label = gtk_label_new (_("File format:"));
  omenu = gtk_option_menu_new ();
  menu = gtk_menu_new ();

  fill_save_menu (GTK_OPTION_MENU (omenu), GTK_MENU (menu));
  
  gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, GNOME_PAD);
  gtk_box_pack_start (GTK_BOX (box), omenu, FALSE, TRUE, GNOME_PAD);
  gtk_widget_show_all (box);
  
  gtk_option_menu_set_menu (GTK_OPTION_MENU (omenu), menu);
  
  return box;
}


void
file_save_as_dialog (gpointer gdata)
{
  DNDData *data = gdata;
  GtkFileSelection *fsel;
  gboolean accepted = FALSE;
  GtkWidget *format_selector;
  
  g_return_if_fail (data != NULL);
  
  fsel = (GtkFileSelection *)gtk_file_selection_new (_("Save workbook as"));
	
  gtk_window_set_modal (GTK_WINDOW (fsel), TRUE);
	
  if (data->file_name)
    gtk_file_selection_set_filename (fsel, data->file_name);
  
  /* -- Choose the format -- */
  format_selector = make_format_chooser ();
  gtk_box_pack_start (GTK_BOX (fsel->action_area), format_selector,
		      FALSE, TRUE, 0);

  /* Connect the signals for Ok and Cancel */
  gtk_signal_connect (GTK_OBJECT (fsel->ok_button), "clicked",
		      GTK_SIGNAL_FUNC (set_ok), &accepted);
  gtk_signal_connect (GTK_OBJECT (fsel->cancel_button), "clicked",
		      GTK_SIGNAL_FUNC (gtk_main_quit), NULL);
  gtk_window_set_position (GTK_WINDOW (fsel), GTK_WIN_POS_MOUSE);
  
  
  /* Run the dialog */
  gtk_widget_show (GTK_WIDGET (fsel));
  gtk_grab_add (GTK_WIDGET (fsel));
  gtk_main ();


  if (accepted)
    {
      char *name = gtk_file_selection_get_filename (fsel);

      data->file_name = g_strdup (name);
      
      current_saver->save (data->mol, data->file_name);
    }

  gtk_widget_destroy (GTK_WIDGET (fsel));

}


void
file_save (gpointer gdata)
{
  DNDData *data = gdata;
  
  g_return_if_fail (data != NULL);
  
  if (!data->file_name)
    {
      file_save_as_dialog (data);
      return;
    }
  
  current_saver->save (data->mol, data->file_name);
  
}


static void
open_file_sel (GtkWidget *widget, gpointer data)
{
  GtkFileSelection *fsel;
  gboolean accepted;
  
  accepted = FALSE;

  fsel = (GtkFileSelection *)gtk_file_selection_new ("Choose a file" );
  gtk_window_set_modal (GTK_WINDOW (fsel), TRUE);

  gtk_signal_connect (GTK_OBJECT (fsel->ok_button ),
		      "clicked", GTK_SIGNAL_FUNC (set_ok), &accepted);
  
  gtk_signal_connect_object (GTK_OBJECT (fsel->cancel_button),
			     "clicked",
			     GTK_SIGNAL_FUNC (gtk_main_quit), NULL);
  
  gtk_window_set_position (GTK_WINDOW (fsel), GTK_WIN_POS_MOUSE);

  /* Run the dialog */
  gtk_widget_show (GTK_WIDGET (fsel));
  gtk_grab_add (GTK_WIDGET (fsel));
  gtk_main ();

  if (accepted)
    {
      gtk_entry_set_text (GTK_ENTRY (data),
			  gtk_file_selection_get_filename (fsel));
    }
  
  gtk_widget_destroy (GTK_WIDGET (fsel));

}


gboolean
file_open_dialog (GtkWidget *widget, gpointer data)
{
  DNDData *d_data = data;
  GtkWidget *dialog;
  GtkWidget *table;
  GtkWidget *entry1, *entry2, *button, *label;
  GtkWidget *omenu;
  GtkWidget *menu;
  GSList    *group;
  gint i;
  int  nbut;
  char *text;
  gboolean ok = FALSE;
  
  dialog = gnome_dialog_new (_("File Open"),
			     GNOME_STOCK_BUTTON_OK,
			     GNOME_STOCK_BUTTON_CANCEL,
			     NULL);

  gnome_dialog_set_parent (GNOME_DIALOG (dialog),
			   GTK_WINDOW (d_data->window->app));

  table = gtk_table_new (3, 2, FALSE);
  
  label = gtk_label_new ("PSF :");
  entry1 = gtk_entry_new();
  button = gtk_button_new_with_label ("Browse...");
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (open_file_sel), entry1);
  gtk_table_attach_defaults (GTK_TABLE (table), label,  0, 1, 0, 1);
  gtk_table_attach_defaults (GTK_TABLE (table), entry1, 1, 2, 0, 1);
  gtk_table_attach_defaults (GTK_TABLE (table), button, 2, 3, 0, 1);
    
  omenu = gtk_option_menu_new ();
  menu       = gtk_menu_new ();
  
  for (group = NULL, i = 0; format_item[i].name; i++)
    {
      GtkWidget *menu_item;
      menu_item = gtk_radio_menu_item_new_with_label (group,
						      format_item[i].name);
      group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (menu_item));
      gtk_menu_append (GTK_MENU (menu), menu_item);
      gtk_widget_show (menu_item);
    }
  
  gtk_option_menu_set_menu (GTK_OPTION_MENU (omenu), menu);
  gtk_option_menu_set_history (GTK_OPTION_MENU (omenu), 0);
  
  entry2 = gtk_entry_new ();
  button = gtk_button_new_with_label ("Browse...");
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (open_file_sel), entry2);

  gtk_table_attach_defaults (GTK_TABLE (table), omenu, 0, 1, 1, 2);
  gtk_table_attach_defaults (GTK_TABLE (table), entry2, 1, 2, 1, 2);
  gtk_table_attach_defaults (GTK_TABLE (table), button, 2, 3, 1, 2);
  
  gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), table,
		      TRUE, TRUE, 0);

  gtk_widget_show_all (table);
  
  /* Run the dialog */
  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
  nbut = gnome_dialog_run (GNOME_DIALOG (dialog));

  if (d_data->psf_name)
    g_free (d_data->psf_name);
  if (d_data->file_name)
    g_free (d_data->file_name);
  
  switch (nbut)
    {
    case 0:
      {
	text = gtk_entry_get_text (GTK_ENTRY (entry1));
	d_data->psf_name = g_strdup (text);
	text = gtk_entry_get_text (GTK_ENTRY (entry2));
	d_data->file_name = g_strdup (text);
	
	i = gtk_radio_menu_group_get_selected (group);
	d_data->file_format = format_item[i].file_format;
	
	ok = TRUE;
	break;
      }
    case 1:
      {
	d_data->file_name = NULL;
	ok = FALSE;
	break;
      }
    }

  gnome_dialog_close (GNOME_DIALOG (dialog));
  
  return ok;
  
}


/*
  From gnumeric... file.c
*/

void
file_format_register_save (char *extension, char *format_description,
			   FileFormatSave save_fn)
{
  FileSaver *fs = g_new (FileSaver, 1);
  
  g_return_if_fail (save_fn != NULL);
	
  fs->extension = extension;
  fs->format_description = format_description;
  fs->save = save_fn;

  dnd_file_savers = g_list_prepend (dnd_file_savers, fs);
  
}


void
file_format_init ()
{
  
  file_format_register_save (".pdb", _("PDB file format"), write_pdb);
  file_format_register_save (".crd", _("CHARMM file format"), write_crd);
    
}
