#include <gnome.h>
#include <gtk/gtk.h>
#include "dnd_main.h"
#include "dnd_command.h"
#include "dnd_menu.h"
#include "dnd_toolbar.h"
#include "dnd_sm.h"
#include "dnd_glarea.h"
#include "dnd_color.h"
#include "dnd_command_area.h"
#include "dnd_filesel.h"

#ifdef DND_USE_MDI

static int
delete_event_handler (GtkWidget *widget, gpointer data)
{
  DNDWindow *window = (DNDWindow *) data;
  
  gnome_config_sync ();

  if (window->properties.save_session)
    gtk_signal_emit_by_name (GTK_OBJECT (window->client), "save_yourself");

  gtk_main_quit ();
	
  return TRUE;
}


#endif

int
dnd_main (int argc, char **argv)
{
  GtkWidget *main_vbox;
  GtkWidget *command_box;
  DNDWindow *window;
  DNDData   *data;
  
  window = g_new0 (DNDWindow, 1);
  window->command = g_strdup (argv[0]);

  data = g_new0 (DNDData, 1);
  data->window = window;
  
   /* -- session management init -- */
  dnd_SM_init (window);
  
  /* make the main window and bind the delete event */
#ifdef DND_USE_MDI

  /* prepare MDI interface */
  window->mdi = GNOME_MDI (gnome_mdi_new ("dnd", "DnD Package"));
  window->mdi->tab_pos = GTK_POS_TOP;

  gtk_signal_connect (GTK_OBJECT (window->mdi), "destroy",
		      GTK_SIGNAL_FUNC (delete_event_handler), window);
  
#else
  window->app = gnome_app_new ("dnd", "DnD Package");
  gtk_widget_realize (window->app);
  gtk_signal_connect (GTK_OBJECT (window->app), "delete_event",
		      GTK_SIGNAL_FUNC (file_quit_cb),
		      NULL);
#endif

  create_dnd_glarea  (data);
  create_dnd_menu (data);
  create_dnd_toolbar (data);
  /*  
  set_dnd_popup_menu (window->app);
  */
#ifdef DND_USE_MDI
  /* set MDI mode */
  gnome_mdi_set_mode (window->mdi, GNOME_MDI_TOPLEVEL);

  /* Restore MDI session. */
  
  if ((gnome_client_get_flags (window->client) & GNOME_CLIENT_RESTORED) ||
      window->properties.save_session) {
    gnome_config_push_prefix
      (gnome_client_get_config_prefix (window->client));
    
    restart_ok = gnome_mdi_restore_state
      (mdi, "MDI Session", gtop_page_create_from_config);
    
    gnome_config_pop_prefix ();
  }

  dnd_mdi_start (!restart_ok);
  
#endif

  command_box = create_dnd_command_area (data);
  
  main_vbox = gtk_vbox_new (FALSE, 0);
  
  gtk_box_pack_start (GTK_BOX (main_vbox), data->glarea, 
		       FALSE, FALSE, 0);

  gtk_box_pack_start (GTK_BOX (main_vbox), command_box,
		      TRUE, TRUE, 0);
  
  color_info_init (&data->color_info);
  file_format_init ();
  
#ifndef DND_USE_MDI  
  gnome_app_set_contents (GNOME_APP (window->app), main_vbox);
  gtk_widget_show_all (main_vbox);
  
  gtk_widget_show (window->app);
#endif
  gtk_widget_grab_focus (GTK_WIDGET (data->glarea));

  return TRUE;
}

