#include <gnome.h>
#include <gtk/gtk.h>
#include "dnd_command.h"
#include "dnd_menu.h"
#include "dnd_preferences.h"

static GtkWidget *dnd_popup_menu;

static GnomeUIInfo help_menu [] = {
  GNOMEUIINFO_ITEM_STOCK (N_("About DND..."), NULL, about_cb, 
			  GNOME_STOCK_MENU_ABOUT),
  GNOMEUIINFO_END
};

static GnomeUIInfo settings_menu [] = {
  /*  GNOMEUIINFO_ITEM_STOCK (N_("Set Color..."), NULL, set_color_cb,
      GNOME_STOCK_MENU_BLANK), */
  GNOMEUIINFO_ITEM_STOCK (N_("Ball and Stick"),
			  N_("Show the Molecule as Ball-and-stick"),
			  setting_graphics_ball_stick_cb,
			  GNOME_STOCK_MENU_BLANK),
  GNOMEUIINFO_ITEM_STOCK (N_("Stick"),
			  N_("Show the Molecule as stick"),
			  setting_graphics_stick_cb,
			  GNOME_STOCK_MENU_BLANK),
  GNOMEUIINFO_ITEM_STOCK (N_("Bond"),
			  N_("Show the Molecule as bond line"),
			  setting_graphics_bond_cb,
			  GNOME_STOCK_MENU_BLANK),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_MENU_PREFERENCES_ITEM( dnd_prefs_dialog, NULL ),
  GNOMEUIINFO_END
};


static GnomeUIInfo plugins_menu [] = {
  GNOMEUIINFO_END
};


static GnomeUIInfo file_menu [] = {
  GNOMEUIINFO_MENU_OPEN_ITEM (file_open_cb, NULL),
  GNOMEUIINFO_MENU_SAVE_ITEM (file_save_cb, NULL),
  GNOMEUIINFO_MENU_SAVE_AS_ITEM(file_save_as_cb, NULL),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_MENU_CLOSE_ITEM(file_close_cb, NULL),
  GNOMEUIINFO_ITEM_STOCK (N_("Exit"), NULL, file_quit_cb,
			  GNOME_STOCK_MENU_EXIT),
  GNOMEUIINFO_END
};


/* The menu definitions: File/Exit and Help/About are mandatory */
static GnomeUIInfo main_menu [] = {
  GNOMEUIINFO_MENU_FILE_TREE (file_menu),
  GNOMEUIINFO_SUBTREE (N_("_Plugins"), plugins_menu),
  GNOMEUIINFO_MENU_SETTINGS_TREE(settings_menu),
  GNOMEUIINFO_MENU_HELP_TREE(help_menu),
  GNOMEUIINFO_END
};


static GnomeUIInfo popup_menu [] = {
  GNOMEUIINFO_ITEM_STOCK (N_("Rotate Mode"),
			  N_("Rotate the molecule"),
			  popup_rotate_cb,
			  GNOME_STOCK_MENU_BLANK),
  GNOMEUIINFO_ITEM_STOCK (N_("Translate Mode"),
			  N_("Transate the molecule"),
			  popup_translate_cb,
			  GNOME_STOCK_MENU_BLANK),
  GNOMEUIINFO_ITEM_STOCK (N_("Zoom Mode"),
			  N_("Zoom the molecule"),
			  popup_zoom_cb,
			  GNOME_STOCK_MENU_BLANK),
  GNOMEUIINFO_ITEM_STOCK (N_("Pick Mode"),
			  N_("Pick the molecule"),
			  popup_pick_cb,
			  GNOME_STOCK_MENU_BLANK),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_ITEM_STOCK (N_("Perspective Projection"),
			  N_("See Object like seeing with your eye"),
			  popup_perspective_cb,
			  GNOME_STOCK_MENU_BLANK),
  GNOMEUIINFO_ITEM_STOCK (N_("Orthographic Projection"),
			  N_("See Object in a box"),
			  popup_orthographic_cb,
			  GNOME_STOCK_MENU_BLANK),
  GNOMEUIINFO_END
};

/*
static void add_dnd_plugin_menu (DNDData *gdata);
*/


void
create_dnd_menu (DNDData *data)
{
  
#ifdef DND_USE_MDI
  gnome_mdi_set_menubar_template (data->window->mdi, main_menu);

  /* and document menu and document list paths */
  gnome_mdi_set_child_menu_path (window->mdi, _("File"));
  gnome_mdi_set_child_list_path (window->mdi, _("Children/"));

#else /* NOT DND_USE_MDI */
  gnome_app_create_menus_with_data (GNOME_APP (data->window->app), 
				    main_menu, 
				    (gpointer) data);
  
  dnd_popup_menu = gnome_popup_menu_new (popup_menu);
  gnome_popup_menu_attach (dnd_popup_menu,
			   data->glarea,
			   data);
  
#endif
}

