#include <gnome.h>
#include "dnd_main.h"
#include "dnd_color.h"
#include "dnd_preferences.h"
#include "dnd_glarea.h"


static GtkWidget *set_color_info (GtkWidget *prop_win, DNDData *data);
static GtkWidget *set_graphic_info (GtkWidget *prop_win, DNDData *data);

static struct 
{
  char *title;
  GtkWidget *(*create_page) (GtkWidget *prop_win, DNDData *data);
} pref_pages [] = {
  { N_("Color"), set_color_info},
  { N_("Graphic"), set_graphic_info},  
  /* { N_("General"), NULL}, */
  { NULL, NULL}
};

typedef struct 
{
  gint color_num;
  GtkWidget *color_label;
  ColorShow *cs;
} PrefColorInfo;


static GtkWidget *pref_prop_win = 0;
static gint  pref_last_page_used = 0;

static void
open_select_color (GtkWidget *widget, gpointer gdata)
{
  PrefColorInfo *pci = gdata;
  gint cn;
  
  cn = dnd_color_selection_dialog ();

  if (cn == -1)
    return;
  
  pci->color_num = cn;
  pci->cs->color = named_colors[cn].color;

  gtk_label_set_text (GTK_LABEL (pci->color_label),
		      named_colors[cn].name);
  
  render (pci->cs);
    
}


static void
pref_properties_apply (GtkObject *w, int page, DNDData *data)
{
  PrefColorInfo *pci;
  
  pci = (PrefColorInfo *)gtk_object_get_data (w, "C");
  data->color_info->c_color = pci->color_num;
  
  pci = (PrefColorInfo *)gtk_object_get_data (w, "N");
  data->color_info->n_color = pci->color_num;
  
  pci = (PrefColorInfo *)gtk_object_get_data (w, "O");
  data->color_info->o_color = pci->color_num;

  pci = (PrefColorInfo *)gtk_object_get_data (w, "H");
  data->color_info->h_color = pci->color_num;
  
  pci = (PrefColorInfo *)gtk_object_get_data (w, "S");
  data->color_info->s_color = pci->color_num;

  if (data->mol)
    {
      delete_display_lists ();

      dnd_display (data);

      gtk_widget_draw (GTK_WIDGET (data->glarea), NULL);
    }
  
}


static void
pref_properties_close (void)
{
  GnomePropertyBox *pbox = GNOME_PROPERTY_BOX (pref_prop_win);
  gtk_main_quit ();
  pref_last_page_used = gtk_notebook_get_current_page (GTK_NOTEBOOK (pbox->notebook));

  gtk_widget_destroy (pref_prop_win);
  pref_prop_win = 0;
}



static void 
add_color_info (GtkWidget *prop_win, GtkWidget *table, gint nrow,
		gint color_num, char *name)
{
  GtkWidget *label = gtk_label_new (name);
  GtkWidget *color_label;
  GtkWidget *button;
  ColorShow *cs = g_new0 (ColorShow, 1);
  PrefColorInfo *pci = g_new0 (PrefColorInfo, 1);
  
  button = dnd_color_selection_button (cs, named_colors[color_num].name);

  pci->color_num = color_num;
  pci->cs = cs;
  
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (open_select_color), pci);
  
  color_label = gtk_label_new (named_colors[color_num].name);
  pci->color_label = color_label;
  
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, nrow, nrow+1);
  gtk_table_attach_defaults (GTK_TABLE (table), color_label, 1, 2, nrow, nrow+1);
  gtk_table_attach (GTK_TABLE (table), button, 2, 3, nrow, nrow+1,
		    GTK_SHRINK, GTK_FILL|GTK_EXPAND, 0, 0);

  gtk_object_set_data (GTK_OBJECT (prop_win), name, pci);
  
}

  
  

static GtkWidget *
set_color_info (GtkWidget *prop_win, DNDData *data)
{
  GtkWidget *main_box;
  GtkWidget *table;
  
  main_box = gtk_vbox_new (FALSE, 0);
  table    = gtk_table_new (3, 5, FALSE);
  gtk_container_add (GTK_CONTAINER (main_box), table);
  
  add_color_info (prop_win, table, 0, data->color_info->c_color, "C");
  add_color_info (prop_win, table, 1, data->color_info->n_color, "N");
  add_color_info (prop_win, table, 2, data->color_info->o_color, "O");
  add_color_info (prop_win, table, 3, data->color_info->h_color, "H");
  add_color_info (prop_win, table, 4, data->color_info->s_color, "S");

  gtk_widget_show (main_box);
  
  return main_box;
  
}


static GtkWidget *
set_graphic_info (GtkWidget *prop_win, DNDData *data)
{
  GtkWidget *main_box;
  
  main_box = gtk_vbox_new (FALSE, 0);

  /* [FIXME] */

  gtk_widget_show (main_box);
  
  return main_box;
    
}



void
dnd_prefs_dialog ( GtkWidget *widget, gpointer gdata )
{
  GtkWidget *prop_win;
  DNDData *data = gdata;
  gint i;
  
  g_return_if_fail (data);
  
  prop_win = gnome_property_box_new ();
  gnome_dialog_set_parent (GNOME_DIALOG (prop_win),
			   GTK_WINDOW (data->window->app));

  for (i = 0; pref_pages[i].title; i++)
    {
      GtkWidget *page;
      GtkWidget *label = gtk_label_new (_(pref_pages[i].title));
      
      page = (*pref_pages[i].create_page) (prop_win, data);
      gnome_property_box_append_page (GNOME_PROPERTY_BOX (prop_win),
				      page, label);
    }
  				     
				     
  gtk_signal_connect (GTK_OBJECT (prop_win), "apply",
		      GTK_SIGNAL_FUNC (pref_properties_apply), data);
  
  gtk_signal_connect (GTK_OBJECT (prop_win), "destroy",
		      GTK_SIGNAL_FUNC (pref_properties_close), NULL);
  

  gtk_notebook_set_page (GTK_NOTEBOOK (GNOME_PROPERTY_BOX (prop_win)->notebook),
			 pref_last_page_used);
  gtk_widget_show_all (prop_win);
  gtk_grab_add (prop_win);
  
  pref_prop_win = prop_win;

  gnome_property_box_changed (GNOME_PROPERTY_BOX (prop_win));
  
  gtk_main ();
  
  pref_prop_win = NULL;
    
}


