#include <GL/gl.h>
#include <GL/glu.h>
#include <math.h>
#include <glib.h>
#include "vector3.h"
#include "ogl_libs.h"

static const vector3 xaxis = {1.0, 0.0, 0.0};
static const vector3 yaxis = {0.0, 1.0, 0.0};

void
ogl_cylinder_faces (vector3 *pos1, vector3 *pos2,
		    double radius, int segments, gboolean capped)
     /*
       Requires mode glShadeModel(GL_SMOOTH) for correct rendering.
     */
{
  int i;
  double angle;
  vector3 point, axis, arad, brad, rad;

  g_return_if_fail (pos1 != NULL);
  g_return_if_fail (pos2 != NULL);
  g_return_if_fail (pos1 != pos2);
  g_return_if_fail (radius > 0.);
  g_return_if_fail (v3_distance (pos1, pos2) > 0.0);
 
  if (segments < 3)
    segments = 3;
  
  v3_difference (&axis, pos2, pos1);
  v3_cross_product (&brad, &axis, &xaxis);
  if (v3_length (&brad) < 1.0e-10)
    v3_cross_product (&brad, &axis, &yaxis);
  v3_cross_product (&arad, &brad, &axis);
  v3_normalize (&arad);
  v3_cross_product (&brad, &arad, &axis);
  v3_normalize (&brad);

  glBegin (GL_QUAD_STRIP);
  glNormal3d (arad.x, arad.y, arad.z);
  
  v3_sum_scaled (&point, pos1, radius, &arad);
  glVertex3d (point.x, point.y, point.z);
  
  v3_sum_scaled (&point, pos2, radius, &arad);
  glVertex3d (point.x, point.y, point.z);
  
  for (i = 1; i < segments; i++) {
    angle = 2.0 * M_PI * (double) i / (double) segments;
    v3_scaled (&rad, cos (angle), &arad);
    v3_add_scaled (&rad, sin (angle), &brad);
    glNormal3d (rad.x, rad.y, rad.z);
    v3_sum_scaled (&point, pos1, radius, &rad);
    glVertex3d (point.x, point.y, point.z);

    v3_sum_scaled (&point, pos2, radius, &rad);
    glVertex3d (point.x, point.y, point.z);
  }
  glNormal3d (arad.x, arad.y, arad.z);
  
  v3_sum_scaled (&point, pos1, radius, &arad);
  glVertex3d (point.x, point.y, point.z);

  v3_sum_scaled (&point, pos2, radius, &arad);
  glVertex3d (point.x, point.y, point.z);

  glEnd();
  /*
  if (capped) {
    v3_normalize (&axis);
    disk (pos2, &axis, &arad, &brad, radius, segments);
    v3_reverse (&axis);
    v3_reverse (&brad);
    disk (pos1, &axis, &arad, &brad, radius, segments);
  }
  */
}
