#ifndef DND_MOLECULE_H
#define DND_MOLECULE_H

#include "vector3.h"
#include <glib.h>

enum { LATTICE, PROTEIN, PDB, CRD };


typedef struct _DMolecule Molecule;
typedef struct _DSegment  Segment;
typedef struct _DResidue  Residue;
typedef struct _DLattice  Lattice;
typedef struct _DAtom     Atom;
typedef struct _vector3   Pos;
typedef struct _DBond     Bond;
typedef struct _DAngle    Angle;
typedef struct _DDihedral Dihedral;

/* -- public -- */

struct _DMolecule
{
  int  type;
  int  model;
  GList *seg_list;
  GList *res_list;
  GList *atom_list;
  GList *pos_list;
  GList *bond_list;
  GList *lattice_list;
};


#define RES_NAME_LENGTH 4
#define RES_TYPE_LENGTH 6

struct _DLattice
{
  char name[2];
};

struct _DSegment
{
  char name[5];
  GList *res_list;
};


struct _DResidue 
{
  char name[RES_NAME_LENGTH+1];
  char type[RES_TYPE_LENGTH+1];
  int  ordinal;
  char chain;
  char code;
  char sec_struc;
  gboolean heterogen;
  double accessibility;
  GList *atom_list;
  /*  colour colour; */
};

#define ATOM_NAME_LENGTH 4
#define ATOM_TYPE_LENGTH 4

struct _DAtom 
{
  gint natom;          /* from 0, ... */ 
  char name[ATOM_NAME_LENGTH+1];
  char type[ATOM_TYPE_LENGTH+1];
  double mass;
  double charge;
  gint8  status;
  int   element;
  int   ordinal;
  double bfactor;
  double occupancy;
  GList *res_list;
  gboolean    is_move;
};


struct _DBond
{
  int    natom[2];
  double force;
};

struct _DAngle
{
  int  natom[3];
};

struct _DDihedral
{
  int  natom[4];
};

/*
Molecule *mol;
*/

/* -- molecule.c -- */
void molecule_free (Molecule **mol);

gboolean read_pdb (Molecule **gmol, char *fname);
gboolean read_psf (Molecule **gmol, char *fname);
gboolean read_molecule (char *fname);
gboolean read_crd (Molecule **gmol, char *fname);
gboolean write_crd (Molecule *mol, char *fname);
gboolean write_pdb (Molecule *mol, char *fname);
void     pos_update (Molecule *mol);


#endif
