#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include "dnd_util.h"
#include "molecule.h"


static void crd_write_title (FILE *fd);
static void crd_write_atom_number (FILE *fd, Molecule *mol);
static void crd_write_atoms (FILE *fd, Molecule *mol);


gboolean
write_crd (Molecule *mol, char *fname)
{
  FILE *crd_file;
  
  g_return_val_if_fail (fname != NULL, FALSE);
  g_return_val_if_fail (mol != NULL, FALSE);
  
  crd_file = fopen (fname, "wt");
  
  if (crd_file == NULL)
    {
      g_warning ("Unable to open .crd file %s ", fname);
      return FALSE;
    }

  crd_write_title (crd_file);
  crd_write_atom_number (crd_file, mol);
  crd_write_atoms (crd_file, mol);

  fclose (crd_file);
  
  return TRUE;
}


static void
crd_write_title (FILE *fd)
{
  fprintf (fd, "* This file is generated by DND \n");
  fprintf (fd, "*                               \n");
}


static void
crd_write_atom_number (FILE *fd, Molecule *mol)
{
  gint natom;
  
  natom = g_list_length (mol->atom_list);
  
  fprintf (fd, "%5d\n", natom);
}


static void
crd_write_atoms (FILE *fd, Molecule *mol)
{
  GList *atom_list;
  GList *pos_list;
  GList *prev_res_list;
  GList *seg_list, *seg_list_next;
  Atom  *atom;
  Pos   *pos;
  Residue *res;
  Segment *seg, *seg_next;
  gint  natom;
  gint  nres;
  
  natom = 0;
  nres  = 0;
  prev_res_list = NULL;
  
  seg_list = mol->seg_list;
  seg = (Segment *)seg_list->data;
  
  seg_list_next = seg_list->next;
  if (seg_list_next)
    seg_next = seg_list_next->data;
  else
    seg_next = NULL;
  
  
  for (atom_list = mol->atom_list, pos_list = mol->pos_list;
       atom_list;
       atom_list = atom_list->next, pos_list = pos_list->next)
    {
      atom = (Atom*) atom_list->data;
      pos  = (Pos *) pos_list->data;
      
      natom++;
  
      if (atom->res_list != prev_res_list)
	{
	  prev_res_list = atom->res_list;
	  res = (Residue *) atom->res_list->data;
	      
	  if (seg_next != NULL)
	    {
	      nres = 0;
	      
	      if (seg_next->res_list == atom->res_list)
		{
		  seg = seg_next;
		  seg_list = seg_list_next;
		  
		  seg_list_next = seg_list->next;
		  
		  if (seg_list_next)
		    seg_next = (Segment*)seg_list_next->data;
		  else
		    seg_next = NULL;
		}
	    }
	  nres++;
	}
      
      
      /* ATOMNO RESNO   RES  TYPE  X     Y     Z   SEGID RESID Weighting */
      fprintf (fd, "%5d%5d %-4s %-4s%10.5f%10.5f%10.5f %-4s %-4s%10.5f \n",
	       natom, nres, res->name, atom->name,
	       pos->x, pos->y, pos->z,
	       seg->name, res->type,
	       atom->bfactor);
    }
  
}
