#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include "dnd_util.h"
#include "molecule.h"

static void pdb_write_title (FILE *fd);
static void pdb_write_atoms (FILE *fd, Molecule *mol);

gboolean 
write_pdb (Molecule *mol, char *fname)
{
  FILE *pdb_file;

  g_return_val_if_fail (fname != NULL, FALSE);
  g_return_val_if_fail (mol != NULL, FALSE);
  
  pdb_file = fopen (fname, "wt");
  
  if (pdb_file == NULL)
    {
      g_warning ("Unable to open .crd file %s ", fname);
      return FALSE;
    }

  pdb_write_title (pdb_file);
  pdb_write_atoms (pdb_file, mol);

  fclose (pdb_file);
  
  return TRUE;
}



static void
pdb_write_title (FILE *fd)
{
  fprintf (fd, "REMARK This file is generated by DND \n");
  fprintf (fd, "REMARK                               \n");
}


static void
pdb_write_atoms (FILE *fd, Molecule *mol)
{
  GList *atom_list;
  GList *pos_list;
  GList *prev_res_list;
  GList *seg_list, *seg_list_next;
  Atom  *atom;
  Pos   *pos;
  Residue *res;
  Segment *seg, *seg_next;
  gint  natom;
  gint  nres;
  
  natom = 0;
  nres  = 0;
  prev_res_list = NULL;
  
  seg_list = mol->seg_list;
  seg = (Segment *)seg_list->data;
  
  seg_list_next = seg_list->next;
  if (seg_list_next)
    seg_next = seg_list_next->data;
  else
    seg_next = NULL;
  
  
  for (atom_list = mol->atom_list, pos_list = mol->pos_list;
       atom_list;
       atom_list = atom_list->next, pos_list = pos_list->next)
    {
      atom = (Atom*) atom_list->data;
      pos  = (Pos *) pos_list->data;
      
      natom++;
  
      if (atom->res_list != prev_res_list)
	{
	  prev_res_list = atom->res_list;
	  res = (Residue *) atom->res_list->data;
	      
	  if (seg_next != NULL)
	    {
	      nres = 0;
	      
	      if (seg_next->res_list == atom->res_list)
		{
		  seg = seg_next;
		  seg_list = seg_list_next;
		  
		  seg_list_next = seg_list->next;
		  
		  if (seg_list_next)
		    seg_next = (Segment*)seg_list_next->data;
		  else
		    seg_next = NULL;
		}
	    }
	  nres++;
	}
      /*
	text IATOM  TYPE  RES  IRES      X  Y  Z  A  W
	A6   I5  2X A4   A4    I5  4X     3F8.3 F6.2 F6.2
      */
      fprintf (fd, "ATOM  %5d  %-4s%-4s%5d    %8.3f%8.3f%8.3f%6.2f%6.2f\n",
	       natom, atom->name, res->name, nres, 
	       pos->x, pos->y, pos->z,
	       atom->occupancy, atom->bfactor);
    }
  
  
}
