#include <math.h>
#include <glib.h>
#include "molecule.h"
#include "md.h"
#include "vector3.h"

void
md_init ()
{

  force_calc ();
  
  md_allocate (data->system);

  /* Initialise velocities */
  md_thermalise (data->system, data->mol, data->control->temper);
    
}


/*
  From Moldy
  md_thermalise  set velocities and quaternion derivatives to values sampled  
  at random from the Boltzmann distribution for the required temperature.
*/

static void
md_thermalise (System *system, Molecule *mol, double temp)
{
  int i;
  int natoms;
  double root_ktm;
  double mass;
  GList *atom_list;
  
  g_return_if_fail (system != NULL);
  g_return_if_fail (mol    != NULL);
  
  natoms = system->natoms;
  
  for (i = 0, atom_list = mol->atom_list; atom_list;
       i++, atom_list = atom_list->next)
    {
      Atom *atom = (Atom *)atom_list->data;

      if (atom->is_move)
	{
	  mass = atom->mass;
          
	  /* [FIXME] */
	  root_ktm = sqrt (kB * temp / mass);
	  
	  vel[i].x = root_ktm * gauss_rand ();
	  vel[i].y = root_ktm * gauss_rand ();
	  vel[i].z = root_ktm * gauss_rand ();
	  
	}
	  
    }
    
}


/******************************************************************************
 *  random number generators. Note they assume 'unsigned' of at least 32 bits *
 ******************************************************************************/
#define  im  1771875L
#define  ia  2416L
#define  ic  374441L

static unsigned long jran = 1;
void	smdrand(unsigned long seed)
{
   jran = seed;
}

double	mdrand()
{
   jran = (jran*ia + ic) % im;
   return (double)jran/(double)im;
}

/*
  gauss_rand. Return a random variable from a gaussian distribution with uni 
  variance.  After Press, Plannery, Teulkolsk & Vetterling p202.             
*/

static double
gauss_rand ()
{
  static int		set = 1;
  static double	gset;
  double		v1, v2, r, fac;
  
  if(set)
    {
      do
	{
	  v1 = 2.0*mdrand()-1.0;  v2 = 2.0*mdrand() - 1.0;
	  r = v1*v1 + v2*v2;
	} while(r > 1.0);
      fac = sqrt(-2.0 * log(r) / r);
      gset = v1 * fac;
      set = 0;
      return(v2*fac);
    }
  else
    {
      set = 1;
      return(gset);
    }
}


