#include <glib.h>
#include <stdio.h>
#include "molecule.h"
#include "rtf.h"
#include "dnd_util.h"
#include "psf.h"

static void pdb2psf_add_atoms (Molecule *mol);

void
pdb2psf (Molecule *mol)
{
  g_return_if_fail (mol != NULL);
  
  pdb2psf_add_atoms (mol);
  psf_add_bonds (mol);
    
}


static void
pdb2psf_add_atoms (Molecule *mol)
{
  GList *atom_list;
  GList *res_list;
  GList *old_res_list;
  Atom   *atom;
  Residue *res;
  RTF_Residue *rtf_res;
  RTF_Atom    *rtf_atom;
  
  res_list = NULL;
  old_res_list = NULL;
  
  for (atom_list = mol->atom_list; atom_list; atom_list = atom_list->next)
    {
      atom = atom_list->data;
      res_list = atom->res_list;
      
      if (res_list != old_res_list)
	{
	  old_res_list = res_list;
	  
	  res = res_list->data;
	  rtf_res = rtf_get_residue (res->name);
	}
      
      rtf_atom = rtf_get_atom (rtf_res, atom->name);
      
      atom->mass = rtf_atom->atom_type->mass;
      atom->charge = rtf_atom->charge;
      strcpy (atom->type, rtf_atom->atom_type->name);
    }
}


