#include <glib.h>
#include <stdio.h>
#include "molecule.h"
#include "rtf.h"
#include "dnd_util.h"
#include "psf.h"

static gint 
psf_get_atom_number (Molecule *mol, GList *res_list, char *at_name);

void
psf_add_bonds (Molecule *mol)
{
  Bond    *bondp;
  GList   *res_list;
  RTF_Residue *rtf_res;
  RTF_Residue *rtf_pept;
  RTF_Bond    *rtf_bond;
  Residue *res;
  gint     ii, jj;
  
  rtf_pept = rtf_get_residue ("PEPT");
  
  for (res_list = mol->res_list; res_list; res_list = res_list->next)
    {
      GList *rtf_bl;
      
      res = res_list->data;
      rtf_res = rtf_get_residue (res->name);
      
      for (rtf_bl = rtf_res->bond_list; rtf_bl; rtf_bl = rtf_bl->next)
	{
	  rtf_bond = rtf_bl->data;
	  
	  ii = psf_get_atom_number (mol, res_list, rtf_bond->at_name[0])+1;
	  jj = psf_get_atom_number (mol, res_list, rtf_bond->at_name[1])+1;
	  
	  if (ii == 0 || jj == 0)
	    continue;
	  
	  bondp = g_new0 (Bond, 1);
	  
	  if (ii < jj)
	    {
	      bondp->natom[0] = ii;
	      bondp->natom[1] = jj;
	    }
	  else
	    {
	      bondp->natom[1] = ii;
	      bondp->natom[0] = jj;
	    }

	  mol->bond_list = g_list_append (mol->bond_list, bondp);
	  
	}
     
      for (rtf_bl = rtf_pept->bond_list; rtf_bl; rtf_bl = rtf_bl->next)
	{
	  rtf_bond = rtf_bl->data;
	  
	  ii = psf_get_atom_number (mol, res_list, rtf_bond->at_name[0])+1;
	  jj = psf_get_atom_number (mol, res_list, rtf_bond->at_name[1])+1;
	  
	  if (ii == 0 || jj == 0)
	    continue;
	  
	  bondp = g_new0 (Bond, 1);
	  
	  if (ii < jj)
	    {
	      bondp->natom[0] = ii;
	      bondp->natom[1] = jj;
	    }
	  else
	    {
	      bondp->natom[1] = ii;
	      bondp->natom[0] = jj;
	    }

	  mol->bond_list = g_list_append (mol->bond_list, bondp);
	  
	}
    }
}



static gint
psf_get_atom_number (Molecule *mol, GList *res_list, char *at_name)
{
  GList *at_list;
  Residue *res;
  Atom    *atom;
  gboolean ok = FALSE;

  
  if (at_name[0] == '-')
    {
      res_list = res_list->prev;
      at_name++;
    }
  else if (at_name[0] == '+')
    {
      /*      res_list = res_list->next; */
      at_name++;
    }

  if (res_list == NULL)
    return -1;
  
  res = res_list->data;
  
  for (at_list = res->atom_list; at_list; at_list = at_list->next)
    {
      atom = at_list->data;
      
      if (atom->res_list != res_list)
	{
	  ok = FALSE;
	  break;
	}
      
      if (str_eq (atom->name, at_name))
	{
	  ok = TRUE;
	  break;
	}
    }
  
  if (ok)
    {
      gint i = g_list_position (mol->atom_list, at_list);
      return i;
    }
  else
    return -1;
}
