#include <stdio.h>
#include <stdlib.h>
#include <glib.h>
#include "dnd_util.h"
#include "molecule.h"

static char buffer[80];
static gboolean read_lattice (FILE *);
static gboolean read_protein (FILE *);

gboolean
read_molecule (char *file_name)
{
  FILE *in_file;
  char type[20];
  gboolean ok;
  
  in_file = fopen (file_name, "r");
  
  if (in_file == NULL)
    {
      g_error ("READ_MOLECULE : Unable to open %s \n", file_name);
      return FALSE;
    }
    
  while (dnd_read_line (in_file, buffer))
    if (!dnd_blank_string (buffer))
      break;
    
  if (str_eq_min ("TYPE", buffer))
    {
      sscanf (buffer+5, "%s", type);
    }
  else
    {
      g_error ("READ_MOLECULE: There is no \"TYPE\" in file %s \n",
	       file_name);
      return FALSE;
    }

  if (str_eq_min (type, "LATTICE"))
    {
      ok = read_lattice (in_file);
    }
  else if (str_eq_min (type, "PROTEIN"))
    {
      ok = read_protein (in_file);
    }

  fclose (in_file);
  
  return ok;
}


static gboolean
read_lattice (FILE *in_file)
{
  int  num;
  char name[2];
  double x, y, z;
  Lattice  *lat;
  Pos      *pos;
  Bond     *bond;
  
  mol = g_new0 (Molecule, 1);
  mol->type = LATTICE;
  
  while (dnd_read_line (in_file, buffer))
    if(!dnd_blank_string (buffer))
      break;

  while (TRUE)
    {
      if (str_eq_min ("ATOM", buffer))
	{
	  sscanf (buffer+5, "%d %s %lf %lf %lf",
		  &num, name, &x, &y, &z);
	  lat = g_new0 (Lattice, 1);
	  
	  strcpy (lat->name, name);
	  mol->lattice_slist = g_slist_append (mol->lattice_slist, lat);
	  
	  pos = g_new0 (Pos, 1);
	  
	  pos->x = x;
	  pos->y = y;
	  pos->z = z;
	  
	  mol->pos_slist = g_slist_append (mol->pos_slist, pos);
	}
      else if (str_eq_min ("BOND", buffer))
	{
	  int ia, ib;
	  
	  sscanf (buffer+5, "%d %d", &ia, &ib);
	  
	  bond = g_new0 (Bond, 1);
	  bond->natom[0] = ia-1;
	  bond->natom[1] = ib-1;
	  mol->bond_slist = g_slist_append (mol->bond_slist, bond);
	}
      else if (str_eq_min ("END", buffer))
	break;
      
     if (! (dnd_read_line (in_file, buffer)))
       break;
      
    }
  
  return TRUE;
  
}


static gboolean
read_protein (FILE *in_file)
{
  
  return TRUE;
}
