#include <stdio.h>
#include <stdlib.h>
#include <glib.h>
#include "dnd_util.h"
#include "molecule.h"
#include "seq.h"

static void seq_read_water (FILE *fd, Molecule *mol, Segment *seg);
static void seq_read_residue (FILE *fd, Molecule *mol, Segment *seg);

void
read_seq (Molecule **gmol, char *fname)
{
  FILE *seq_file;
  Segment  *seg;
  Molecule *mol;
  char     buffer[200];
  char     seg_name[10];
  gint     i;
  
  g_return_if_fail (fname != NULL);
  g_return_if_fail (gmol  != NULL);
  
  mol = *gmol;
  
  if (mol != NULL)
    {
      g_error ("Read_SEQ : molecule is not NULL\n");
      molecule_free (&mol);
    }
  
  mol = g_new0 (Molecule, 1);

  seq_file = fopen (fname, "rt");
  
  g_return_if_fail (seq_file != NULL);
  
  while (dnd_read_line (seq_file, buffer))
    {
      if (dnd_blank_string (buffer))
	continue;
      
      if (!dnd_find_word (buffer, "SEQ"))
	{
	  g_message ("Unable to find \"SEQ\" string in psf file %s\n",
		     fname);
	  return;
	}
      else
	break;
    }
  
  while (dnd_read_line (seq_file, buffer))
    {
      g_strup (buffer);
      
      if (dnd_blank_string (buffer))
	continue;

      if (dnd_find_word (buffer, "END"))
	break;
      
      if (!dnd_find_word (buffer, "SEG"))
	{
	  g_message ("Unable to find \"SEG\" string in psf file %s\n",
		     fname);
	  return;
	}
     
      i = dnd_index (buffer, "SEG")+1;
  
      sscanf (buffer+i, "%*s %s", seg_name);

      seg = g_new0 (Segment, 1);
      strcpy (seg->name, seg_name);
      mol->seg_list = g_list_append (mol->seg_list, seg);
  
      if (str_eq_min (seg_name, "WAT"))
	seq_read_water (seq_file, mol, seg);
      else
	seq_read_residue (seq_file, mol, seg);
    }
  
  *gmol = mol;
  
}


static void
seq_read_water (FILE *fd, Molecule *mol, Segment *seg)
{
  char res_name[10];
  char buffer[200];
  GList *res_list = g_list_last (mol->res_list);
  Residue *res;
  gint  i;
  gint  nwat;
  
  while (dnd_read_line (fd, buffer))
    {
      if (dnd_blank_string (buffer))
	continue;
      else
	break;
    }
  
  sscanf (buffer, "%s %d", res_name, &nwat);
  
  for (i = 0; i < nwat; i++)
    {
      res = g_new0 (Residue, 1);
      strcpy (res->name, res_name);

      if (res_list == NULL)
	{
	  mol->res_list = g_list_append (mol->res_list, res);
	  res_list = mol->res_list;
	}
      else
	{
	  res_list = g_list_append (res_list, res);
	  res_list = res_list->next;
	}

      if (seg->res_list == NULL)
	seg->res_list = res_list;
    }
  
}

static void
seq_read_residue (FILE *fd, Molecule *mol, Segment *seg)
{
  char *buf;
  char res_name[100];
  char buffer[200];
  gint nres, num_res;
  Residue *res;
  GList *res_list = g_list_last (mol->res_list);

  /* --- Read Residue Number --- */  
  while (dnd_read_line (fd, buffer))
    {
      if (dnd_blank_string (buffer))
	continue;
      
      sscanf (buffer, "%d", &num_res);
      break;
    }
  

  /* --- read residue --- */
  nres = 0;
  
  while (dnd_read_line (fd, buffer))
    {
      if (dnd_blank_string (buffer))
	continue;

      buf = buffer;
      
      while (buf && (nres < num_res))
	{
	  buf = dnd_next_word (buf, res_name);
	  res = g_new0 (Residue, 1);
	  strcpy (res->name, res_name);

	  if (res_list == NULL)
	    {
	      mol->res_list = g_list_append (mol->res_list, res);
	      res_list = mol->res_list;
	    }
	  else
	    {
	      res_list = g_list_append (res_list, res);
	      res_list = res_list->next;
	    }

	  if (seg->res_list == NULL)
	    seg->res_list = res_list;
	  
	  nres++;
	}
      
      if (nres == num_res)
	break;
    }

}

