#include <stdio.h>
#include <glib.h>
#include "molecule.h"
#include "dnd_util.h"
#include "rtf.h"
#include "psf.h"
#include "seq.h"
#include "md.h"


static void run_read (MDData *data, char *command_line);
static void run_write (MDData *data, char *command_line);
static void run_generate (MDData *data, char *command_line);

void
run_commands (MDData *data)
{
  GSList *slist;
  char *command_line;
  char word[100];
  
  g_return_if_fail (data != NULL);
  g_return_if_fail (data->command_line != NULL);
  

  for (slist = data->command_line; slist; slist = slist->next)
    {
      command_line = slist->data;
  
      command_line = dnd_next_word (command_line, word);
      
      if (word[0] == '!' || word[0] == '#')
	continue;
      
      g_strup (word);
      
      if (str_eq_min (word, "READ"))
	{
	  run_read (data, command_line);
	}
      else if (str_eq_min (word, "WRITE"))
	{
	  run_write (data, command_line);
	}
      else if (str_eq_min (word, "GENE"))
	{
	  run_generate (data, command_line);
	}
    }
  
}


void
run_read (MDData *data, char *command_line)
{
  char *buf;
  char word[100];
  
  buf = dnd_next_word (command_line, word);

  g_strup (word);
  
  if (str_eq_min (word, "RTF"))
    {
      sscanf (buf, "%s", word);
      read_rtf (word);
    }
  else if (str_eq_min (word, "PSF"))
    {
      sscanf (buf, "%s", word);
      read_psf (&data->mol, word);
    }
  else if (str_eq_min (word, "SEQ"))
    {
      sscanf (buf, "%s", word);
      read_seq (&data->mol, word);
    }
  else if (str_eq_min (word, "PDB"))
    {
      sscanf (buf, "%s", word);
      read_pdb (&data->mol, word);
    }
}

void
run_write (MDData *data, char *command_line)
{
  char *buf;
  char word[100];

  buf = dnd_next_word (command_line, word);
  
  g_strup (word);
  
  if (str_eq_min (word, "PSF"))
    {
      sscanf (buf, "%s", word);
      write_psf (data->mol, word);
    }
}


void
run_generate (MDData *data, char *command_line)
{
  char *buf;
  char word[100];
  gint i, j;
  
  g_strup (command_line);
  
  buf = dnd_next_word (command_line, word);
  
  if (str_eq_min (word, "PSF"))
    {
      i = dnd_index (buf, "from") + 1;
      j = dnd_index (buf+i, " ") + 1;
      sscanf (buf+i+j, "%s", word);
      
      if (str_eq_min (word, "PDB"))
	{
	  pdb2psf (data->mol);
	}
      else if (str_eq_min (word, "SEQ"))
	{
	  seq2psf (data->mol);
	}
    }
}

  
  
