#include <glib.h>
#include <stdio.h>
#include "molecule.h"
#include "rtf.h"
#include "dnd_util.h"
#include "psf.h"
#include "seq.h"

static void seq2psf_add_atoms (Molecule *mol);

void
seq2psf (Molecule *mol)
{
  g_return_if_fail (mol != NULL);
  
  seq2psf_add_atoms (mol);
  psf_add_bonds (mol);
  
}


static void
seq2psf_add_atoms (Molecule *mol)
{
  GList *atom_list;
  GList *rtf_atom_list;
  GList *res_list;
  Atom  *atom;
  Residue *res;
  RTF_Residue *rtf_res;
  RTF_Residue *rtf_pept;
  RTF_Atom    *rtf_atom;
  
  if (mol->atom_list != NULL)
    {
      g_list_free (atom_list);
      mol->atom_list = NULL;
    }
  
  atom_list = NULL;

  rtf_pept = rtf_get_residue ("PEPT");
  
  for (res_list = mol->res_list; res_list; res_list = res_list->next)
    {
      res = res_list->data;
      
      rtf_res = rtf_get_residue (res->name);

      for (rtf_atom_list = rtf_res->atom_list;
	   rtf_atom_list; rtf_atom_list = rtf_atom_list->next)
	{
	  rtf_atom = rtf_atom_list->data;
      
	  atom = g_new0 (Atom, 1);
      
	  strcpy (atom->name, rtf_atom->name);
	  strcpy (atom->type, rtf_atom->atom_type->name);
	  atom->mass = rtf_atom->atom_type->mass;
	  atom->charge = rtf_atom->charge;
	  atom->res_list = res_list;
	  
	  if (atom_list == NULL)
	    {
	      atom_list = g_list_append (atom_list, atom);
	      mol->atom_list = atom_list;
	    }
	  else
	    {
	      atom_list = g_list_append (atom_list, atom);
	      atom_list = atom_list->next;
	    }

	  if (res->atom_list == NULL)
	    res->atom_list = atom_list;
	  
	}

    }
  
}

