#include <stdio.h>
#include <stdlib.h>
#include <glib.h>
#include "dnd_util.h"
#include "molecule.h"
#include "psf.h"

static void psf_write_remark (FILE *fd, Molecule *mol);
static void psf_write_atoms  (FILE *fd, Molecule *mol);
static void psf_write_bonds  (FILE *fd, Molecule *mol);

  

void
write_psf (Molecule *mol, char *fname)
{
  FILE *psf_file;
  
  g_return_if_fail (fname != NULL);
  g_return_if_fail (mol != NULL);
  
  psf_file = fopen (fname, "wt");
  
  if (psf_file == NULL)
    {
      g_message ("Write_Psf : Unable to open .psf file %s\n", fname);
      return;
    }
  
  psf_write_remark (psf_file, mol);
  psf_write_atoms  (psf_file, mol);
  psf_write_bonds  (psf_file, mol);
  
  fclose (psf_file);
  
}


static void
psf_write_remark (FILE *fd, Molecule *mol)
{
  gint i = 1;
  
  fprintf (fd, "PSF\n");
  fprintf (fd, "%8d !NTITLE\n", i);
  fprintf (fd, " REMARKS this file is generated by DND package\n");
  fprintf (fd, "\n");
}


static void
psf_write_atoms (FILE *fd, Molecule *mol)
{
  GList *seg_list;
  GList *seg_list_next;
  GList *ls;
  GList *res_list;
  GList *old_res_list;
  Atom  *atom;
  Residue *res;
  Segment *seg, *seg_next;
  gint num_atoms;
  gint natom;
  gint nres;
  gint zero = 0;
    
  num_atoms = g_list_length (mol->atom_list);
  
  fprintf (fd,"%8d ! NATOM \n", num_atoms);
  
  natom = 0;
  nres  = 0;
  
  /* Initiate SEGMENT */
  seg_list = mol->seg_list;
  seg = seg_list->data;

  seg_list_next = seg_list->next;
  
  if (seg_list_next)
    seg_next = seg_list_next->data;
  else
    seg_next = NULL;
  
  for (ls = mol->atom_list; ls; ls = ls->next)
    {
      atom = ls->data;
      natom++;
      
      res_list = atom->res_list;
      
      if (res_list != old_res_list)
	{
	  old_res_list = res_list;
	  nres++;
	  
	  res = res_list->data;
	  
	  if (seg_next)
	    {
	      if (seg_next->res_list == res_list)
		{
		  seg_list = seg_list_next;
		  seg = seg_next;
		  seg_list_next = seg_list->next;
		  if (seg_list_next)
		    seg_next = seg_list_next->data;
		  else
		    seg_next = NULL;
		  
		  nres = 1;
		}
	    }
	}
      
      if (atom->charge < 1e-5 && atom->charge > -1e-5)
	fprintf (fd, "%8d %-4s %-4d %-4s %-4s %-5s  %10.6E   %-10.4f       %d \n",
		 natom, seg->name, nres, res->name, atom->name, atom->type,
		 atom->charge, atom->mass, zero);
      else if (atom->charge < -1e-5)
	fprintf (fd, "%8d %-4s %-4d %-4s %-4s %-5s %-10.6f      %-10.4f       %d \n",
		 natom, seg->name, nres, res->name, atom->name, atom->type,
		 atom->charge, atom->mass, zero);
      else
	fprintf (fd, "%8d %-4s %-4d %-4s %-4s %-5s  %-10.6f     %-10.4f       %d \n",
		 natom, seg->name, nres, res->name, atom->name, atom->type,
		 atom->charge, atom->mass, zero);
    }
  
  fprintf (fd, "\n");
  
}


static void
psf_write_bonds (FILE *fd, Molecule *mol)
{
  GList *ls;
  Bond  *bondp;
  gint num_bonds;
  gint nb;
  
  num_bonds = g_list_length (mol->bond_list);
  
  fprintf (fd, "%8d ! NBOND \n", num_bonds);
  
  nb = 0;
  
  for (ls = mol->bond_list; ls; ls = ls->next)
    {
      bondp = ls->data;
      nb++;
      
      fprintf (fd, "%8d%8d", bondp->natom[0], bondp->natom[1]);
      
      if ((nb%4) == 0)
	{
	  fprintf (fd, "\n");
	}
    }
 
  fprintf (fd,"\n");
  
}
  

