#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include "dnd_util.h"
#include "molecule.h"

static void del_water (Molecule *s_mol, Molecule *w_mol);
static void add_water (Molecule *s_mol, Molecule *w_mol);
static GList *del_water_1 (Molecule *w_mol, Pos *pos);

gint
main (int argc, char **argv)
{
  Molecule *s_mol;
  Molecule *w_mol;
  Molecule *c_mol;
  
  if (argc < 4)
    {
      g_print ("[del_wat] [solutes file] [water file] [created file] \n");
      return 1;
    }
  s_mol = w_mol = c_mol = NULL;
  
  read_crd (&s_mol, argv[1]);
  read_crd (&w_mol, argv[2]);

  g_print ("solute atoms %d, water atoms %d , water residue %d\n",
	   g_list_length(s_mol->atom_list),
	   g_list_length (w_mol->atom_list),
	   g_list_length (w_mol->res_list));
  
  /* run */
  del_water (s_mol, w_mol);

  add_water (s_mol, w_mol);
  
  g_print ("solute atoms %d, water atoms %d \n",
	   g_list_length(s_mol->atom_list),
	   g_list_length (w_mol->atom_list));
  g_print ("water residue %d, created residue %d\n",
	    g_list_length (w_mol->res_list),
	    g_list_length(s_mol->res_list));
  
  write_crd (s_mol, argv[3]);

  return 1;
  
}



static void
del_water (Molecule *s_mol, Molecule *w_mol)
{
  GList    *s_list;
  GList    *w_list;
  vector3  *s_pos, *w_pos;
  double    dis;

  for (s_list = s_mol->pos_list; s_list; s_list = s_list->next)
    {
      s_pos = s_list->data;
      
      for (w_list = w_mol->pos_list; w_list; )
	{
	  w_pos = w_list->data;
	  dis = v3_distance(s_pos, w_pos);
	  
	  if (dis < 2.0)
	    {
	      w_list = del_water_1 (w_mol, w_pos);
	    }
	  else
	    w_list = w_list->next;
	}
    }
}



static void 
add_water (Molecule *s_mol, Molecule *w_mol)
{
  GList *list;
  GList *tlist;
    
  list = g_list_last (s_mol->atom_list);
  
  for (tlist = w_mol->atom_list; tlist; tlist = tlist->next)
    {
      Atom *atom = tlist->data;
      g_list_append (list, atom);
      list = list->next;
    }
  
  list = g_list_last (s_mol->pos_list);
  
  for (tlist = w_mol->pos_list; tlist; tlist = tlist->next)
    {
      Pos *pos = tlist->data;
      g_list_append (list, pos);
      list = list->next;
    }
  
  list = g_list_last (s_mol->seg_list);
  
  for (tlist = w_mol->seg_list; tlist; tlist = tlist->next)
    {
      Segment *seg = tlist->data;
      g_list_append (list, seg);
      list = list->next;
    }
  
  list = g_list_last (s_mol->res_list);
  
  for (tlist = w_mol->res_list; tlist; tlist = tlist->next)
    {
      Residue *res = tlist->data;
      g_list_append (list, res);
      list = list->next;
    }
  
}


GList *
del_water_1 (Molecule *w_mol, vector3 *w_pos)
{
  int nth;
  GList *atom_list;
  GList *res_list, *res_list_next;
  GList *prev_atom_list, *prev_res_list, *prev_pos_list;
  GList *pos_list;
  GList *list;
  Atom  *atom;
  Residue *res;
  Pos   *pos;
  
  nth = g_list_index (w_mol->pos_list, w_pos);
  
  /* [FIXME] */
  atom = (Atom *)g_list_nth_data (w_mol->atom_list, nth);
  res_list  = atom->res_list;
  res = res_list->data;

  /* -- First atom_list of Res --- */
  atom_list = res->atom_list;
  atom = atom_list->data;
  prev_atom_list = atom_list->prev;
  
  /* --- First pos_list of Res --- */
  nth = g_list_position (w_mol->atom_list, atom_list);
  pos_list = g_list_nth (w_mol->pos_list, nth);
  prev_pos_list = pos_list->prev;
  
  /* --- */
  res->atom_list = NULL;
  res_list_next = res_list->next;
    
  while (atom->res_list != res_list_next)
    {
      /* -- Free atom_list -- */
      atom->res_list = NULL;
      g_free (atom);
      
      list = atom_list->next;
      g_list_remove_link (prev_atom_list, atom_list);
      g_list_free_1 (atom_list);
      
      atom_list = list;
      atom = atom_list->data;
      
      /* --- Free pos_list --- */
      pos = pos_list->data;
      g_free (pos);
      list = pos_list->next;
      g_list_remove_link (prev_pos_list, pos_list);
      g_list_free_1 (pos_list);
      
      pos_list = list;
    }
  
  g_free (res);
  
  prev_res_list = res_list->prev;
  
  g_list_remove_link (prev_res_list, res_list);
  g_list_free_1 (res_list);

  g_print (" delete ... water \n");
  
  return prev_pos_list;
  
}

